#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file VelocitySwitcher.py
 @brief Filter for TimedVelocity2D, changes output according to Game State and Camera Recognition Result.
 @date $Date$


"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist

import VelocitySwitcher_idl

# Import Service implementation class
# <rtc-template block="service_impl">
from VelocitySwitcher_idl_example import *

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
velocityswitcher_spec = ["implementation_id", "VelocitySwitcher", 
		 "type_name",         "VelocitySwitcher", 
		 "description",       "Filter for TimedVelocity2D, changes output according to Game State and Camera Recognition Result.", 
		 "version",           "1.0.0", 
		 "vendor",            "KanamuraRobotics", 
		 "category",          "Controller", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

##
# @class VelocitySwitcher
# @brief Filter for TimedVelocity2D, changes output according to Game State and Camera Recognition Result.
# 
# 
class VelocitySwitcher(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		self._d_vel_in = OpenRTM_aist.instantiateDataType(RTC.TimedVelocity2D)
		"""
		"""
		self._vel_inIn = OpenRTM_aist.InPort("vel_in", self._d_vel_in)
		self._d_vel_out = OpenRTM_aist.instantiateDataType(RTC.TimedVelocity2D)
		"""
		"""
		self._vel_outOut = OpenRTM_aist.OutPort("vel_out", self._d_vel_out)

		"""
		"""
		self._gameStateSwitcherPort = OpenRTM_aist.CorbaPort("gameStateSwitcher")
		"""
		"""
		self._colorSwitcherPort = OpenRTM_aist.CorbaPort("colorSwitcher")

		"""
		"""
		self._gameStateSwitcherService = VelocitySwitcherService_i()
                self._gameStateSwitcherService.setSwitchCallback(self._gameStateSwitched)
		"""
		"""
		self._colorSwitcherService = VelocitySwitcherService_i()
		self._colorSwitcherService.setSwitchCallback(self._colorStateSwitched)


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		
		# </rtc-template>

        def _colorStateSwitched(self, flag):
                print("Color State")
                if flag == False:
                        print("your lose.")
                        time.sleep(60)
			print("hello.")
                
                pass

        def _gameStateSwitched(self, flag):
                print("Game State")
                if flag == False:
                        print("im dead.")
                        time.sleep(3600)
                
                pass
		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		# Bind variables and configuration variable
		
		# Set InPort buffers
		self.addInPort("vel_in",self._vel_inIn)
		
		# Set OutPort buffers
		self.addOutPort("vel_out",self._vel_outOut)
		
		# Set service provider to Ports
		self._gameStateSwitcherPort.registerProvider("KanamuraRobotics::VelocitySwitcherService", "VelocitySwitcherService", self._gameStateSwitcherService)
		self._colorSwitcherPort.registerProvider("KanamuraRobotics::VelocitySwitcherService", "VelocitySwitcherService", self._colorSwitcherService)
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		self.addPort(self._gameStateSwitcherPort)
		self.addPort(self._colorSwitcherPort)
		
		return RTC.RTC_OK
	
	###
	## 
	## The finalize action (on ALIVE->END transition)
	## formaer rtc_exiting_entry()
	## 
	## @return RTC::ReturnCode_t
	#
	## 
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The startup action when ExecutionContext startup
	## former rtc_starting_entry()
	## 
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The shutdown action when ExecutionContext stop
	## former rtc_stopping_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The activated action (Active state entry action)
	## former rtc_active_entry()
	##
	## @param ec_id target ExecutionContext Id
	## 
	## @return RTC::ReturnCode_t
	##
	##
	#def onActivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The deactivated action (Active state exit action)
	## former rtc_active_exit()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onDeactivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The execution action that is invoked periodically
	## former rtc_active_do()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	def onExecute(self, ec_id):
                if self._vel_inIn.isNew():

                        self._d_vel_out.data = self._vel_inIn.read().data

                        print(self._d_vel_out.data)
                        OpenRTM_aist.setTimestamp(self._d_vel_out)
                        self._vel_outOut.write()
	
		return RTC.RTC_OK
	
	###
	##
	## The aborting action when main logic error occurred.
	## former rtc_aborting_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The error action in ERROR state
	## former rtc_error_do()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The reset action that is invoked resetting
	## This is same but different the former rtc_init_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The state update action that is invoked after onExecute() action
	## no corresponding operation exists in OpenRTm-aist-0.2.0
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##

	##
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The action that is invoked when execution context's rate is changed
	## no corresponding operation exists in OpenRTm-aist-0.2.0
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def VelocitySwitcherInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=velocityswitcher_spec)
    manager.registerFactory(profile,
                            VelocitySwitcher,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    VelocitySwitcherInit(manager)

    # Create a component
    comp = manager.createComponent("VelocitySwitcher")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

